//******************************************************************************
//Bibilioteka obsluga wyswietlacza ze sterownikiem HD44780
//******************************************************************************

#include "lcd_hd44780_lib.h"
#include "stm32f10x_gpio.h"

extern void delay(uint32_t us);

GPIO_InitTypeDef GPIO_InitStructure;

//-----------------------------------------------------------------------------
void LCD_WriteNibble(unsigned char nibbleToWrite) //wpisywanie danych do wyswietlacza
{
  GPIO_WriteBit(LCD_GPIO, LCD_EN, Bit_SET);
	if(nibbleToWrite & 0x01) 
	{
		GPIO_WriteBit(LCD_GPIO, LCD_D4, Bit_SET);
	} else 
	{
		GPIO_WriteBit(LCD_GPIO, LCD_D4, Bit_RESET);
	}
  if(nibbleToWrite & 0x02) 
	{
		GPIO_WriteBit(LCD_GPIO, LCD_D5, Bit_SET);
	}
	else
	{
		GPIO_WriteBit(LCD_GPIO, LCD_D5, Bit_RESET);
	}
  if(nibbleToWrite & 0x04)
	{
		GPIO_WriteBit(LCD_GPIO, LCD_D6, Bit_SET);
	} 
	else
	{
		GPIO_WriteBit(LCD_GPIO, LCD_D6, Bit_RESET);
	}
  if(nibbleToWrite & 0x08)
	{
		GPIO_WriteBit(LCD_GPIO, LCD_D7, Bit_SET);
	} else
	{
		GPIO_WriteBit(LCD_GPIO, LCD_D7, Bit_RESET);
	}
  GPIO_WriteBit(LCD_GPIO, LCD_EN, Bit_RESET);
}


//-----------------------------------------------------------------------------
unsigned char LCD_ReadNibble(void)  //zczytywanie danych z wyswietlacza
{
  unsigned char tmp = 0;
  GPIO_WriteBit(LCD_GPIO, LCD_EN, Bit_SET);
  tmp |= (GPIO_ReadInputDataBit(LCD_GPIO, LCD_D4) << 0);
  tmp |= (GPIO_ReadInputDataBit(LCD_GPIO, LCD_D5) << 1);
  tmp |= (GPIO_ReadInputDataBit(LCD_GPIO, LCD_D6) << 2);
  tmp |= (GPIO_ReadInputDataBit(LCD_GPIO, LCD_D7) << 3);
  GPIO_WriteBit(LCD_GPIO, LCD_EN, Bit_RESET);
  return tmp;
}


//-----------------------------------------------------------------------------
unsigned char LCD_ReadStatus(void) //odczyt statusu nieuzywana w STM32 ze wzgledu
	//na poziomy napiec
{
  unsigned char status = 0;
  
  GPIO_InitStructure.GPIO_Pin   =  LCD_D4 | LCD_D5 | LCD_D6 | LCD_D7;
  GPIO_InitStructure.GPIO_Mode  =  GPIO_Mode_IPU;
  GPIO_Init(LCD_GPIO, &GPIO_InitStructure);
  
  GPIO_WriteBit(LCD_GPIO, LCD_RW, Bit_SET);
  GPIO_WriteBit(LCD_GPIO, LCD_RS, Bit_RESET);
  
  status |= (LCD_ReadNibble() << 4);
  status |= LCD_ReadNibble();
  
  GPIO_InitStructure.GPIO_Pin   =  LCD_D4 | LCD_D5 | LCD_D6 | LCD_D7;
  GPIO_InitStructure.GPIO_Mode  =  GPIO_Mode_Out_PP;
  GPIO_Init(LCD_GPIO, &GPIO_InitStructure);
  
  return status;
}


//-----------------------------------------------------------------------------
void LCD_WriteData(unsigned char dataToWrite)
{
  GPIO_WriteBit(LCD_GPIO, LCD_RW, Bit_RESET);
  GPIO_WriteBit(LCD_GPIO, LCD_RS, Bit_SET);
  
  LCD_WriteNibble(dataToWrite >> 4);
  LCD_WriteNibble(dataToWrite & 0x0F);
  
  while(LCD_ReadStatus() & 0x80);
}


//-----------------------------------------------------------------------------
void LCD_WriteCommand(unsigned char commandToWrite) //wyslanie danej komendy
{
  GPIO_WriteBit(LCD_GPIO, LCD_RW | LCD_RS, Bit_RESET);
  LCD_WriteNibble(commandToWrite >> 4);
  LCD_WriteNibble(commandToWrite & 0x0F);
  
  while(LCD_ReadStatus() & 0x80);
}


//-----------------------------------------------------------------------------
void LCD_WriteText(unsigned char * text)
{
  while(*text)
    LCD_WriteData(*text++);
}


//-----------------------------------------------------------------------------
void LCD_GoTo(unsigned char x, unsigned char y)
	//funkcja zmieniajaca polozenie kursora na wyswietlaczu
{
  LCD_WriteCommand(HD44780_DDRAM_SET | (x + (0x40 * y)));
}


//-----------------------------------------------------------------------------
void LCD_WriteTextXY(char text[], unsigned char x, unsigned char y) //funkcja sluzaca do
	//wyswietlania tekstu na wyswietlaczu
{
	char *s;
	s=text;
  LCD_GoTo(x,y);
  while(*s)  
	{
		LCD_WriteData(*s);
			s++;
	}
}


//-----------------------------------------------------------------------------
void LCD_WriteBinary(unsigned int var, unsigned char bitCount)
{
  signed char i;
  
  for(i = (bitCount - 1); i >= 0; i--)
     {
     LCD_WriteData((var & (1 << i))?'1':'0');
     }
}


//-----------------------------------------------------------------------------
void LCD_ShiftLeft(void)
{
  LCD_WriteCommand(HD44780_DISPLAY_CURSOR_SHIFT | HD44780_SHIFT_LEFT | HD44780_SHIFT_DISPLAY);
}


//-----------------------------------------------------------------------------
void LCD_ShiftRight(void)
{
  LCD_WriteCommand(HD44780_DISPLAY_CURSOR_SHIFT | HD44780_SHIFT_RIGHT | HD44780_SHIFT_DISPLAY);
}


//-----------------------------------------------------------------------------
void LCD_Initialize(void)
{
 // uint16_t k=1; //zmienna k ustawiana jako mnoznik dla wartosci zmienne us czyli TIMERUS
  //konfiguracja wyjsc dla wyswietlacza HD44780
  GPIO_InitStructure.GPIO_Pin   =  LCD_D4|LCD_D5|LCD_D6|LCD_D7|LCD_RS|LCD_RW|LCD_EN;
  GPIO_InitStructure.GPIO_Speed =  GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode  =  GPIO_Mode_Out_PP;
  
  GPIO_Init(LCD_GPIO, &GPIO_InitStructure);
  
  GPIO_ResetBits(LCD_GPIO, LCD_RS | LCD_EN | LCD_RW);
  
delay(2000);  //>czekaj 40ms mikrosekund po wzroscie napiecia powyzej 4,5V
 
    LCD_WriteNibble(0x03);            
   
	delay(200); //czekaj >4100us=4,1ms
	
		LCD_WriteNibble(0x03);
	
  delay(5); //czekaj >100 us
	
	LCD_WriteNibble(0x03);
	
	delay(5); //czekaj >100us
	
  LCD_WriteNibble(0x02);
	
		delay(1);
					 

     
  LCD_WriteCommand(HD44780_FUNCTION_SET | 
                   HD44780_FONT5x7 | 
                   HD44780_TWO_LINE | 
                   HD44780_4_BIT);
  
  LCD_WriteCommand(HD44780_DISPLAY_ONOFF | 
                   HD44780_DISPLAY_OFF); 
  
  LCD_WriteCommand(HD44780_CLEAR); 
  
	
	
	//ostatnie wpisanie entry mode
  LCD_WriteCommand(HD44780_ENTRY_MODE | 
                   HD44780_EM_SHIFT_CURSOR | 
                   HD44780_EM_INCREMENT);
  
	//**************************************************************
	//wpisujemy opcje przy jakich bedzie pracowal wyswietlacz ze sterownikiem HD44780
  LCD_WriteCommand(HD44780_DISPLAY_ONOFF | 
                   HD44780_DISPLAY_ON |
                   HD44780_CURSOR_OFF | 
                   HD44780_CURSOR_NOBLINK);
}
//********************************************************************8

//-----------------------------------------------------------------------------
void LCD_SetUserChar (unsigned char chrNum, unsigned char n, const unsigned char *p)
{         //chrNum  - character number (code) to be registered (0..7)
          //n       - number of characters to register
          //*p      - pointer to the character pattern (8 * n bytes)
	LCD_WriteCommand(HD44780_CGRAM_SET | chrNum * 8);
	n *= 8;
	do
		LCD_WriteData(*p++);
	while (--n);
}


//-----------------------------------------------------------------------------
